function output = getTireForces_dugoff(t,S,P,delta)
% A fucntion to compute the tire forces by the Dugoff model.
%=====Input=====%
%t          double          simulation time
%S          struct          vehicle states of the 7-DOF model
%P          struct          Parameters of the vehicle and simulation
%=====Output=====%
%output     struct          A struct containing the longitudinal and
%                           lateral forces of the tires.

%% Read states from S
x_dot = S(1);
y_dot = S(2);
psi = S(3);
psi_dot = S(4);
w_fl = S(5);
w_fr = S(6);
w_rl = S(7);
w_rr = S(8);
X = S(9);
Y = S(10);

%% Read parameters from P
l_f = P.l_f;
l_r = P.l_r;
r_eff = P.r_eff;
C_alpha = P.C_alpha;
C_sigma = P.C_sigma;
mu = P.mu;
%%%%%%%delta = P.delta_profile(t);
F_z = P.m*P.g/4; %Assuming uniform load on each tire.

%% Prob. 1.1a
% Compute Slip Angle

alpha_fl = delta - (y_dot+l_f*psi_dot)/x_dot;
alpha_fr = alpha_fl;
alpha_rl = -(y_dot-l_r*psi_dot)/x_dot;
alpha_rr = alpha_rl;

alpha_list = [alpha_fl, alpha_fr, alpha_rl, alpha_rr];

%% Prob. 1.1b
% Compute Slip Ratio
w_list = [w_fl, w_fr, w_rl, w_rr];
sigma_list = NaN*ones(1,4);
for i = 1:length(w_list)
    w = w_list(i);
	
    sigma_list(i) = r_eff*w - x_dot;
    if sigma_list(i) >= 0
        % Acceleration
        sigma_list(i) = sigma_list(i)/x_dot;
    else
        % Braking
        sigma_list(i) = sigma_list(i)/(r_eff*w);
    end
end
sigma_fl = sigma_list(1);
sigma_fr = sigma_list(2);
sigma_rl = sigma_list(3);
sigma_rr = sigma_list(4);

%% Prob 1.1c
% Compute tire forces
F_x_list = NaN*ones(1,length(w_list));
F_y_list = NaN*ones(1,length(w_list));

for i = 1:length(w_list)
    %Get alpha and sigma for the specific tire
    alpha = alpha_list(i);
    sigma = sigma_list(i);
    
    %Compute lambda
    lambda = (mu*F_z*(1+sigma))/2/sqrt((C_sigma*sigma)^2 + ...
             (C_alpha*tan(alpha))^2);
    
    %Compute f(lambda)
    if lambda < 1
        f_lambda = (2-lambda)*lambda;
    else
        f_lambda = 1;
    end
    
    %Compute tire force
    F_x_list(i) = C_sigma * sigma/(1+sigma) * f_lambda;
    F_y_list(i) = C_alpha * tan(alpha)/(1+sigma) * f_lambda;
end


%% Organize output
output.F_xfl = F_x_list(1);
output.F_xfr = F_x_list(2);
output.F_xrl = F_x_list(3);
output.F_xrr = F_x_list(4);

output.F_yfl = F_y_list(1);
output.F_yfr = F_y_list(2);
output.F_yrl = F_y_list(3);
output.F_yrr = F_y_list(4);
end