function output = vehicle_controller_new(t,S,P)
% A function to compute the yaw-moment controller
%=====Input=====
%t          double          simulation time
%S          struct          vehicle states of the 7-DOF model
%P          struct          Parameters of the vehicle and simulation
%=====Output=====
%output     struct          A structure containing the driving torque
%                           and breaking torque on the wheels.

%% Get parameters from P
l_f = P.l_f;
l_r = P.l_r;
l_w = P.l_w;
C_alpha = P.C_alpha;
m = P.m;
mu = P.mu;
g = P.g;

r_eff = P.r_eff;
rho = P.rho;
xi = P.xi;
eta = P.eta;
I_z = P.I_z;
X_des = P.X_profile(t);
Y_des = P.Y_profile(t);
Yaw_des = P.Yaw_profile(t);
VX_des = P.VX_profile(t);
VY_des = P.VY_profile(t);
R_des = P.R_profile(t);
%%%%%%%%delta = P.delta_profile(t);


%% Get states from S
x_dot = S(1);
y_dot = S(2);
psi = S(3);
psi_dot = S(4);
w_fl = S(5);
w_fr = S(6);
w_rl = S(7);
w_rr = S(8);
X = S(9);
Y = S(10);
x_dot_int_err = S(11);


%% Get steering angle
kp_delta_yaw = 0;%.5;
kp_delta_pos = 1;%.3;
R = R_des;
yaw_err = mod(Yaw_des-psi+pi, 2*pi)-pi;
X_err = X_des-X;
Y_err = Y_des-Y;
pos_adjust = atan2(VY_des, VX_des) - atan2(x_dot*sin(psi)+y_dot*cos(psi), x_dot*cos(psi)-y_dot*sin(psi));
pos_adjust = mod(pos_adjust+pi, 2*pi)-pi;
if R == 0
    delta_start = 0;
else
    delta_start = (l_f+l_r+(m*x_dot^2*(l_r-l_f)*C_alpha)/(2*C_alpha^2*(l_f+l_r)))/R;
end
delta_yaw_corr = kp_delta_yaw*yaw_err;
delta_loc_corr = kp_delta_pos*pos_adjust;
delta = delta_start + delta_yaw_corr + delta_loc_corr;


%% Get throttle for speed control
k_speed_adj = 0.02;
speed_adjust = dot([X_err, Y_err], [x_dot*cos(psi)-y_dot*sin(psi), x_dot*sin(psi)+y_dot*cos(psi)]);
x_dot_des = norm([VX_des, VY_des]) + k_speed_adj*speed_adjust;
x_dot_err = x_dot_des-x_dot;
kp = 2;
ki = 0;%.01;
x_ddot_des = kp*(x_dot_err) + ki*x_dot_int_err;
T_des = (x_ddot_des*m/2)*r_eff;


%% Get tire forces from tire force model
tireForces = getTireForces_dugoff(t,S,P,delta);
F_xfl = tireForces.F_xfl;
F_xfr = tireForces.F_xfr;
F_xrl = tireForces.F_xrl;
F_xrr = tireForces.F_xrr;
F_yfl = tireForces.F_yfl;
F_yfr = tireForces.F_yfr;
F_yrl = tireForces.F_yrl;
F_yrr = tireForces.F_yrr;

%% Compute V and beta from x_dot and y_dot
V = sqrt(x_dot^2 + y_dot^2);
beta = atan2(y_dot,x_dot);

%% Prob. 1.5a
% Compute R according to steady state steering angle
% Choose delta from R instead
%%%R = (l_f+l_r+(m*x_dot^2*(l_r-l_f)*C_alpha)/(2*C_alpha^2*(l_f+l_r)))/delta;

%% Prob. 1.5b
% Compute desired yaw-rate
psi_dot_des = x_dot/R;
% Compute desired side-slip angle
beta_des = l_r/R - (l_f*m*x_dot^2)/(2*C_alpha*(l_f+l_r)*R);


%% Prob. 1.5c
% Compute target psi_dot
psi_dot_upper = 0.85*mu*g/x_dot;
if abs(psi_dot_des) <= psi_dot_upper
    psi_dot_target = psi_dot_des;
else
    psi_dot_target = psi_dot_upper*sign(psi_dot_des);
end

% Compute target beta
beta_upper = atan(0.02*mu*g);
if abs(beta_des) <= beta_upper
    beta_target = beta_des;
else
    beta_target = beta_upper*sign(beta_des);
end

%% Compute s value for the sliding control
s = (psi_dot - psi_dot_target) + xi*(beta-beta_target);

%% Compute the breaking yaw moment M_psi_b
psi_ddot_target = 0;    %for making a fixed radius turn in fixed speed.
beta_dot_target = 0;    %for making a fixed radius turn in fixed speed.

%% Compute beta_dot using current state and parameters
beta_dot = getBetaDot(t,S,P,delta);

%% Prob 1.5d
% Compute desired breaking moment M_psi_b
M_psi_b = (-l_f/I_z*(F_yfl+F_yfr)*cos(delta) -eta*s-xi*beta_dot) ...
          * I_z/(rho+cos(delta));

%% Prob 1.5e
% Compute the actual break torque using the lower level controller
delta_F_xf = 2*M_psi_b/l_w;
if M_psi_b > 0
    T_bfl = delta_F_xf*r_eff;
    T_bfr = 0;
    T_brl = rho*delta_F_xf*r_eff;
    T_brr = 0;
elseif M_psi_b < 0
    T_bfl = 0;
    T_bfr = -delta_F_xf*r_eff;
    T_brl = 0;
    T_brr = -rho*delta_F_xf*r_eff;
else
    T_bfl = 0;
    T_bfr = 0;
    T_brl = 0;
    T_brr = 0;
end
    

%% Organize Output
output.T_dfl = T_des;%%%%%%P.T_d_profile(t);
output.T_dfr = T_des;%%%%%%P.T_d_profile(t);
output.T_drl = 0;
output.T_drr = 0;

output.T_bfl = 0;%T_bfl; %Disable yaw moment control
output.T_bfr = 0;%T_bfr;
output.T_brl = 0;%T_brl;
output.T_brr = 0;%T_brr;

output.delta = delta;
output.x_dot_des = x_dot_des;

end