%A script to simulate the yaw-moment control with 7-DOF vehicle model
clearvars -except controller_select track
if ~exist('controller_select','var')
    controller_select = 3;
end
if ~exist('track','var')
    track = 1;
end

%% Setup Parameters
P.l = 2.5;                  % Car length [m]
P.l_f = 0.4*P.l;            % CoG front axle distance [m]
P.l_r = P.l - P.l_f;        % CoG rear axle distance [m]
P.l_w = 1.539;
P.m = 1274;                 % vehicle sprung mass [Kg]
P.I_z = 1523;               % yaw moment of inertia [kgm^2]
P.g = 9.81;                 % Gravity
P.mu = 0.9;                 % friction coefficient
P.C_alpha = 5500;           % Cornering Stiffness [N/rad]
P.C_sigma = 3000;           % Tire Stiffness []
P.r_eff = 0.305;            % Effective raduis of tire.
P.J_w = 1;                  % Rotational Inertia of Tire
P.rho = 0.25;               % Front/Rear Break ratio.
P.eta = 15;                 % Sliding Control Parameter: time constant of the sliding surface
P.xi = 0.01;                % Sliding Control Parameter: importance of beta.

%% Determine initial condition
P.V_init = 10;              % Desire speed [m/s]
r = 20;

%% Encode Steering Angle profile
%%%%%P.delta_profile = @(t) 0.1*sin((2*pi/50/2)*t);
f = P.V_init/(2*pi*r);  %Hz
dt = 0.1;
P.X_profile = @(t) r.*(cos(2*pi*f.*t)+1).*(1.-2.*(mod(t*f+0.5,2)>1));
P.Y_profile = @(t) r.*sin(2*pi*f.*t);
P.Yaw_profile = @(t) pi/2+pi.*(2.*mod(t.*f-0.5,1)-1).*((mod(t-1/2/f,2/f)-mod(t-1/2/f,1/f)).*2.*f-1);
P.VX_profile = @(t) (P.X_profile(t+dt)-P.X_profile(t))/dt;%-r.*(2*pi*f*sin(2*pi*f.*t)).*(1.-2.*(mod(t*f+0.5,2)>1));
P.VY_profile = @(t) (P.Y_profile(t+dt)-P.Y_profile(t))/dt;%r.*2*pi*f*cos(2*pi*f.*t);
P.R_profile = @(t) r.*(-1+2*(mod(t*f-0.5, 2)>=1));

if controller_select == 1
    P.T_d_profile = @(t) 120;  %For the vehicle_controller_none
elseif controller_select == 2
    P.T_d_profile = @(t) 200;  %For the vehicle_controller
else
    P.T_d_profile = @(t) 480;  %For the vehicle_controller_new
end

%% Set Initial Condition
w_init = P.V_init/P.r_eff;
%S = [x_dot, y_dot, psi, psi_dot, w_fl, w_fr, w_rl, w_rr, X, Y];
P.S_init = [P.V_init, 0, pi/2, 0, w_init, w_init, w_init, w_init, 2*r, 0, 0];
P.tSpan = 0:0.01:80;

%% Simulate the system
if controller_select == 1
    odeFunc = @(t,S) vehicle_plant(t,S,vehicle_controller_none(t,S,P),P);
elseif controller_select == 2
    odeFunc = @(t,S) vehicle_plant(t,S,vehicle_controller(t,S,P),P);
else
    odeFunc = @(t,S) vehicle_plant(t,S,vehicle_controller_new(t,S,P),P);
end

%% Use ode45
[tOut,SOut] = ode45(odeFunc,P.tSpan,P.S_init);

%% Compute beta and V from the output
beta_list = atan2(SOut(:,2),SOut(:,1));
V_list = sqrt(SOut(:,1).^2 + SOut(:,2).^2);
delta_list = zeros(size(tOut));

%% Get delta_list from the soved states
for i = 1:size(SOut, 1)
    control = vehicle_controller_new(tOut(i), SOut(i,:), P);
    delta_list(i) = control.delta;
end

l_f = P.l_f;
l_r = P.l_r;
m = P.m;
C_alpha = P.C_alpha;

R_by_delta = (l_f + l_r + m*V_list.^2*(l_r*C_alpha - l_f*C_alpha)/(2*C_alpha*C_alpha*(l_f+l_r)))./delta_list;
psi_dot_des = V_list./R_by_delta;
beta_des = l_r./R_by_delta - l_r*m*V_list.^2./(2*C_alpha*(l_f+l_r)*R_by_delta);

s_list = SOut(:,4) - psi_dot_des + P.xi*(beta_list-beta_des);


%% Visualize
figure(1);
subplot(3,3,1);
plot(tOut,SOut(:,1),tOut,SOut(:,2),tOut,V_list);
legend('x_{dot}','y_{dot}','V');
subplot(3,3,2)
plot(tOut,beta_list,tOut,beta_des);
legend('\beta','\beta_{des}');
subplot(3,3,3);
plot(tOut,SOut(:,5),tOut,SOut(:,6),tOut,SOut(:,7),tOut,SOut(:,8));
legend('\omega_{fl}','\omega_{fr}','\omega_{rl}','\omega_{rr}');
subplot(3,3,4);
plot(tOut,SOut(:,4),tOut,psi_dot_des);
legend('\psi_{dot}','\psi_{dot,des}');
subplot(3,3,5)
plot(tOut,s_list);
legend('s');
subplot(3,3,7);
plot(tOut,delta_list);
xlabel('time');
ylabel('steering angle');
subplot(3,3,8);
plot(tOut,P.Yaw_profile(tOut));
xlabel('time');
ylabel('desired yaw angle');
subplot(3,3,9);
plot(tOut,P.R_profile(tOut));
xlabel('time');
ylabel('radius of curvature');

figure(2);
subplot(1,1,1);
plot(P.X_profile(tOut),P.Y_profile(tOut),'r--',SOut(:,9),SOut(:,10));
axis equal;
xlabel('X [m]');
ylabel('Y [m]');
legend('Desired', 'Actual');

figure(3);
subplot(1,1,1);
plot(tOut,psi_dot_des,'r--',tOut,SOut(:,4));
xlabel('Time [s]');
ylabel('Yaw rate, \psi_{dot} [rad/s]');
legend('Desired', 'Actual');

figure(4);
subplot(1,1,1);
plot(tOut,P.Yaw_profile(tOut),'r--',tOut,SOut(:,3));
xlabel('Time [s]');
ylabel('Yaw angle, \psi [rad]');
legend('Desired', 'Actual');

figure(5);
subplot(1,1,1);
plot(tOut,SOut(:,1),tOut,SOut(:,2),tOut,V_list);
legend('x_{dot}','y_{dot}','V');
xlabel('Time [s]');
ylabel('Speed [m/s]');

figure(6);
subplot(1,1,1);
plot(tOut,delta_list);
xlabel('Time [s]');
ylabel('Steering angle [rad]');

figure(7);
subplot(1,1,1);
plot(tOut,SOut(:,5),tOut,SOut(:,7));
legend('\omega_{fl}, \omega_{fr}','\omega_{rl}, \omega_{rr}');
xlabel('Time [s]');
ylabel('Wheel speed [rad/s]');


%%
type getTireForces_dugoff;
type vehicle_plant;

%type getBetaDot;
%type vehicle_controller;






